@@ -13,14 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Miscellaneous utility functions for use with Swift."""
+
 from calendar import timegm
-import collections
+from collections.abc import Mapping
 import gzip
 import hashlib
 import hmac
+import io
 import json
 import logging
-import six
 import time
 import traceback
 
@@ -39,7 +40,7 @@ def config_true_value(value):
     This function comes from swift.common.utils.config_true_value()
     """
     return value is True or \
-        (isinstance(value, six.string_types) and value.lower() in TRUE_VALUES)
+        (isinstance(value, str) and value.lower() in TRUE_VALUES)
 
 
 def prt_bytes(num_bytes, human_flag):
@@ -69,12 +70,12 @@ def prt_bytes(num_bytes, human_flag):
 
 
 def generate_temp_url(path, seconds, key, method, absolute=False,
-                      prefix=False, iso8601=False):
+                      prefix=False, iso8601=False, ip_range=None):
     """Generates a temporary URL that gives unauthenticated access to the
     Swift object.
 
     :param path: The full path to the Swift object or prefix if
-         a prefix-based temporary URL should be generated. Example:
+        a prefix-based temporary URL should be generated. Example:
         /v1/AUTH_account/c/o or /v1/AUTH_account/c/prefix.
     :param seconds: time in seconds or ISO 8601 timestamp.
         If absolute is False and this is the string representation of an
@@ -92,6 +93,8 @@ def generate_temp_url(path, seconds, key, method, absolute=False,
     :param prefix: if True then a prefix-based temporary URL will be generated.
     :param iso8601: if True, a URL containing an ISO 8601 UTC timestamp
         instead of a UNIX timestamp will be created.
+    :param ip_range: if a valid ip range, restricts the temporary URL to the
+        range of ips.
     :raises ValueError: if timestamp or path is not in valid format.
     :return: the path portion of a temporary URL
     """
@@ -129,7 +132,7 @@ def generate_temp_url(path, seconds, key, method, absolute=False,
     except ValueError:
         raise ValueError(TIME_ERRMSG)
 
-    if isinstance(path, six.binary_type):
+    if isinstance(path, bytes):
         try:
             path_for_body = path.decode('utf-8')
         except UnicodeDecodeError:
@@ -155,11 +158,24 @@ def generate_temp_url(path, seconds, key, method, absolute=False,
         expiration = int(time.time() + timestamp)
     else:
         expiration = timestamp
-    hmac_body = u'\n'.join([method.upper(), str(expiration),
-                            ('prefix:' if prefix else '') + path_for_body])
+
+    hmac_parts = [method.upper(), str(expiration),
+                  ('prefix:' if prefix else '') + path_for_body]
+
+    if ip_range:
+        if isinstance(ip_range, bytes):
+            try:
+                ip_range = ip_range.decode('utf-8')
+            except UnicodeDecodeError:
+                raise ValueError(
+                    'ip_range must be representable as UTF-8'
+                )
+        hmac_parts.insert(0, "ip=%s" % ip_range)
+
+    hmac_body = '\n'.join(hmac_parts)
 
     # Encode to UTF-8 for py3 compatibility
-    if not isinstance(key, six.binary_type):
+    if not isinstance(key, bytes):
         key = key.encode('utf-8')
     sig = hmac.new(key, hmac_body.encode('utf-8'), hashlib.sha1).hexdigest()
 
@@ -167,12 +183,16 @@ def generate_temp_url(path, seconds, key, method, absolute=False,
         expiration = time.strftime(
             EXPIRES_ISO8601_FORMAT, time.gmtime(expiration))
 
-    temp_url = u'{path}?temp_url_sig={sig}&temp_url_expires={exp}'.format(
+    temp_url = '{path}?temp_url_sig={sig}&temp_url_expires={exp}'.format(
         path=path_for_body, sig=sig, exp=expiration)
+
+    if ip_range:
+        temp_url += '&temp_url_ip_range={}'.format(ip_range)
+
     if prefix:
-        temp_url += u'&temp_url_prefix={}'.format(parts[4])
+        temp_url += '&temp_url_prefix={}'.format(parts[4])
     # Have return type match path from caller
-    if isinstance(path, six.binary_type):
+    if isinstance(path, bytes):
         return temp_url.encode('utf-8')
     else:
         return temp_url
@@ -180,7 +200,7 @@ def generate_temp_url(path, seconds, key, method, absolute=False,
 
 def get_body(headers, body):
     if headers.get('content-encoding') == 'gzip':
-        with gzip.GzipFile(fileobj=six.BytesIO(body), mode='r') as gz:
+        with gzip.GzipFile(fileobj=io.BytesIO(body), mode='r') as gz:
             nbody = gz.read()
         return nbody
     return body
@@ -199,10 +219,10 @@ def parse_api_response(headers, body):
 
 def split_request_headers(options, prefix=''):
     headers = {}
-    if isinstance(options, collections.Mapping):
+    if isinstance(options, Mapping):
         options = options.items()
     for item in options:
-        if isinstance(item, six.string_types):
+        if isinstance(item, str):
             if ':' not in item:
                 raise ValueError(
                     "Metadata parameter %s must contain a ':'.\n"
@@ -234,7 +254,7 @@ def report_traceback():
         return None, None
 
 
-class NoopMD5(object):
+class NoopMD5:
     def __init__(self, *a, **kw):
         pass
 
@@ -245,7 +265,7 @@ class NoopMD5(object):
         return ''
 
 
-class ReadableToIterable(object):
+class ReadableToIterable:
     """
     Wrap a filelike object and act as an iterator.
 
@@ -294,7 +314,7 @@ class ReadableToIterable(object):
         return self
 
 
-class LengthWrapper(object):
+class LengthWrapper:
     """
     Wrap a filelike object with a maximum length.
 
@@ -379,8 +399,28 @@ def n_groups(seq, n):
 
 
 def normalize_manifest_path(path):
-    if six.PY2 and isinstance(path, six.text_type):
-        path = path.encode('utf-8')
     if path.startswith('/'):
         return path[1:]
     return path
+
+
+class JSONableIterable(list):
+    def __init__(self, iterable):
+        self._iterable = iter(iterable)
+        try:
+            self._peeked = next(self._iterable)
+            self._has_items = True
+        except StopIteration:
+            self._peeked = None
+            self._has_items = False
+
+    def __bool__(self):
+        return self._has_items
+
+    __nonzero__ = __bool__
+
+    def __iter__(self):
+        if self._has_items:
+            yield self._peeked
+        for item in self._iterable:
+            yield item
